@@ -63,9 +63,9 @@ class PropertyDelegate(QItemDelegate):
 
         # pixmaps for curve icons
         self.curve_pixmaps = {
-            openshot.BEZIER: QPixmap(":/curves/keyframe-%s.png" % openshot.BEZIER),
-            openshot.LINEAR: QPixmap(":/curves/keyframe-%s.png" % openshot.LINEAR),
-            openshot.CONSTANT: QPixmap(":/curves/keyframe-%s.png" % openshot.CONSTANT),
+            openshot.BEZIER: QIcon(":/curves/keyframe-%s.png" % openshot.BEZIER).pixmap(20, 20),
+            openshot.LINEAR: QIcon(":/curves/keyframe-%s.png" % openshot.LINEAR).pixmap(20, 20),
+            openshot.CONSTANT: QIcon(":/curves/keyframe-%s.png" % openshot.CONSTANT).pixmap(20, 20)
             }
 
     def paint(self, painter, option, index):
@@ -192,18 +192,30 @@ class PropertiesTableView(QTableView):
             self.selected_item = model.item(row, 1)
 
         # Is the user dragging on the value column
-        if self.selected_label and self.selected_item:
+        if self.selected_label and self.selected_item and \
+                self.selected_label.data() and type(self.selected_label.data()) == tuple:
+            # Ignore undo/redo history temporarily (to avoid a huge pile of undo/redo history)
+            get_app().updates.ignore_history = True
+
+            # Disable video caching during drag operation (for performance reasons)
+            openshot.Settings.Instance().ENABLE_PLAYBACK_CACHING = False
+            log.debug('mouseMoveEvent: Stop caching frames on timeline')
+
             # Get the position of the cursor and % value
             value_column_x = self.columnViewportPosition(1)
             cursor_value = event.x() - value_column_x
             cursor_value_percent = cursor_value / self.columnWidth(1)
 
+            # Get data from selected item
             try:
                 cur_property = self.selected_label.data()
             except Exception:
-                # If item is deleted during this drag... an exception can occur
-                # Just ignore, since this is harmless
                 log.debug('Failed to access data on selected label widget')
+                return
+
+            if type(cur_property) != tuple:
+                log.debug('Failed to access valid data on current selected label widget')
+                return
 
             property_key = cur_property[0]
             property_name = cur_property[1]["name"]
@@ -219,9 +231,6 @@ class PropertiesTableView(QTableView):
 
             # Get the original data of this item (prior to any updates, for the undo/redo system)
             if not self.original_data:
-                # Ignore undo/redo history temporarily (to avoid a huge pile of undo/redo history)
-                get_app().updates.ignore_history = True
-
                 # Find this clip
                 c = None
                 if item_type == "clip":
@@ -295,6 +304,10 @@ class PropertiesTableView(QTableView):
         event.accept()
         get_app().updates.ignore_history = False
 
+        # Enable video caching again
+        openshot.Settings.Instance().ENABLE_PLAYBACK_CACHING = True
+        log.debug('mouseReleaseEvent: Start caching frames on timeline')
+
         # Add final update to undo/redo history
         get_app().updates.apply_last_action_to_history(self.original_data)
 
@@ -332,7 +345,7 @@ class PropertiesTableView(QTableView):
         selected_label = model.item(row, 0)
         self.selected_item = model.item(row, 1)
 
-        if selected_label:
+        if selected_label and selected_label.data() and type(selected_label.data()) == tuple:
             cur_property = selected_label.data()
             property_type = cur_property[1]["type"]
 
@@ -343,7 +356,7 @@ class PropertiesTableView(QTableView):
                 blue = cur_property[1]["blue"]["value"]
 
                 # Show color dialog
-                currentColor = QColor(red, green, blue)
+                currentColor = QColor(int(red), int(green), int(blue))
                 log.debug("Launching ColorPicker for %s", currentColor.name())
                 ColorPicker(
                     currentColor, parent=self, title=_("Select a Color"),
@@ -398,9 +411,12 @@ class PropertiesTableView(QTableView):
     def filter_changed(self, value=None):
         """ Filter the list of properties """
 
-        # Update model
+        # Update property model (and re-trigger filter logic)
         self.clip_properties_model.update_model(value)
 
+        # Filter keyframes visible on timeline
+        get_app().window.SetKeyframeFilter.emit(value)
+
     def contextMenuEvent(self, event):
         """ Display context menu """
         # Get property being acted on
@@ -421,9 +437,14 @@ class PropertiesTableView(QTableView):
         _ = get_app()._tr
 
         # If item selected
-        if selected_label:
-            # Get data from selected item
+        if selected_label and selected_label.data() and type(selected_label.data()) == tuple:
             cur_property = selected_label.data()
+
+            # Clear menu if models updated
+            if self.menu_reset:
+                self.choices = []
+                self.menu_reset = False
+
             property_name = cur_property[1]["name"]
             self.property_type = cur_property[1]["type"]
             points = cur_property[1]["points"]
@@ -433,63 +454,49 @@ class PropertiesTableView(QTableView):
             log.info("Context menu shown for %s (%s) for clip %s on frame %s" % (property_name, property_key, clip_id, frame_number))
             log.info("Points: %s" % points)
 
-            # Clear menu if models updated
-            if self.menu_reset:
-                self.choices = []
-                self.menu_reset = False
-
             # Handle parent effect options
             if property_key == "parent_effect_id" and not self.choices:
-                clip_choices = [{
-                    "name": "None",
-                    "value": "None",
-                    "selected": False,
-                    "icon": QIcon()
-                }]
-                # Instantiate the timeline
-                timeline_instance = get_app().window.timeline_sync.timeline
                 # Instantiate this effect
-                effect = timeline_instance.GetClipEffect(clip_id)
-                effect_json = json.loads(effect.Json())
+                effect = Effect.get(id=clip_id)
 
                 # Loop through timeline's clips
-                for clip_instance in timeline_instance.Clips():
-                    clip_instance_id = clip_instance.Id()
-                    # Avoid parent a clip effect to it's own effect
-                    if (clip_instance_id != effect.ParentClipId()):
-                        # Clip's propertyJSON data
-                        clip_instance_data = Clip.get(id = clip_instance_id).data
-                        # Path to the clip file
-                        clip_instance_path = clip_instance_data["reader"]["path"]
-                        # Iterate through all clip files on the timeline
-                        for clip_number in range(self.files_model.rowCount()):
-                            clip_index = self.files_model.index(clip_number, 0)
-                            clip_name = clip_index.sibling(clip_number, 1).data()
-                            clip_path = os.path.join(clip_index.sibling(clip_number, 4).data(), clip_name)
-                            # Check if the timeline's clip file name matches the clip the user selected
-                            if (clip_path == clip_instance_path):
-                                # Generate the clip icon to show in the selection menu
-                                clip_instance_icon = clip_index.data(Qt.DecorationRole)
-                        effect_choices = [{"name": "None",
-                                            "value": "None",
-                                            "selected": False,
-                                            "icon": QIcon()}]
+                clip_choices = []
+                for clip in Clip.filter():
+                    file_id = clip.data.get("file_id")
+
+                    # Look up parent clip id (if effect)
+                    parent_clip_id = effect.parent.get("id")
+
+                    # Avoid attach a clip to it's own object
+                    if clip.id != parent_clip_id:
+                        # Iterate through all project files (to find matching QIcon)
+                        for file_index in range(self.files_model.rowCount()):
+                            file_row = self.files_model.index(file_index, 0)
+                            project_file_id = file_row.sibling(file_index, 5).data()
+                            if file_id == project_file_id:
+                                clip_instance_icon = file_row.data(Qt.DecorationRole)
+                                break
+
+                        effect_choices = []
                         # Iterate through clip's effects
-                        for effect_data in clip_instance_data["effects"]:
+                        for clip_effect_data in clip.data["effects"]:
                             # Make sure the user can only set a parent effect of the same type as this effect
-                            if effect_data['class_name'] == effect_json['class_name']:
-                                effect_id = effect_data["id"]
-                                effect_name = effect_data['class_name']
-                                effect_icon = QIcon(QPixmap(os.path.join(info.PATH, "effects", "icons", "%s.png" % effect_data['class_name'].lower())))
+                            if clip_effect_data['class_name'] == effect.data['class_name']:
+                                effect_id = clip_effect_data["id"]
+                                effect_icon = QIcon(QPixmap(os.path.join(info.PATH, "effects", "icons", "%s.png" % clip_effect_data['class_name'].lower())))
                                 effect_choices.append({"name": effect_id,
                                                 "value": effect_id,
                                                 "selected": False,
                                                 "icon": effect_icon})
-                        self.choices.append({"name": _(clip_instance_data["title"]),
-                                            "value": effect_choices,
-                                            "selected": False,
-                                            "icon": clip_instance_icon})
+                        if effect_choices:
+                            clip_choices.append({"name": _(clip.data["title"]),
+                                                "value": effect_choices,
+                                                "selected": False,
+                                                "icon": clip_instance_icon})
 
+                self.choices.append({"name": _("None"), "value": "None", "selected": False, "icon": None})
+                if clip_choices:
+                    self.choices.append({"name": _("Clips"), "value": clip_choices, "selected": False, "icon": None})
 
             # Handle selected object options (ObjectDetection effect)
             if property_key == "selected_object_index" and not self.choices:
@@ -506,83 +513,38 @@ class PropertiesTableView(QTableView):
                                 "name": str(object_index),
                                 "value": str(object_index),
                                 "selected": False,
-                                "icon": QIcon()
+                                "icon": None
                             })
-                self.choices.append({"name": _("Detected Objects"), "value": object_index_choices, "selected": False, "icon": None})
-
-            # Handle property to set the Tracked Object's child clip
-            if property_key == "child_clip_id" and not self.choices:
-                clip_choices = [{
-                    "name": "None",
-                    "value": "None",
-                    "selected": False,
-                    "icon": QIcon()
-                }]
-                # Instantiate the timeline
-                timeline_instance = get_app().window.timeline_sync.timeline
-                current_effect = timeline_instance.GetClipEffect(clip_id)
-
-                # Loop through timeline's clips
-                for clip_instance in timeline_instance.Clips():
-                    clip_instance_id = clip_instance.Id()
-
-                    # Avoid attach a clip to it's own object
-                    if (clip_instance_id != clip_id and (current_effect and clip_instance_id != current_effect.ParentClip().Id())):
-                        # Clip's propertyJSON data
-                        clip_instance_data = Clip.get(id = clip_instance_id).data
-                        # Path to the clip file
-                        clip_instance_path = clip_instance_data["reader"]["path"]
-                        # Iterate through all clip files on the timeline
-                        for clip_number in range(self.files_model.rowCount()):
-                            clip_index = self.files_model.index(clip_number, 0)
-                            clip_name = clip_index.sibling(clip_number, 1).data()
-                            clip_path = os.path.join(clip_index.sibling(clip_number, 4).data(), clip_name)
-                            # Check if the timeline's clip file name matches the clip the user selected
-                            if (clip_path == clip_instance_path):
-                                # Generate the clip icon to show in the selection menu
-                                clip_instance_icon = clip_index.data(Qt.DecorationRole)
-                                self.choices.append({"name": clip_instance_data["title"],
-                                              "value": clip_instance_id,
-                                              "selected": False,
-                                              "icon": clip_instance_icon})
+                if object_index_choices:
+                    self.choices.append({"name": _("Detected Objects"), "value": object_index_choices, "selected": False, "icon": None})
 
             # Handle clip attach options
-            if property_key == "parentObjectId" and not self.choices:
+            if property_key in ["parentObjectId", "child_clip_id"] and not self.choices:
                 # Add all Clips as choices - initialize with None
-                tracked_choices = [{
-                    "name": "None",
-                    "value": "None",
-                    "selected": False,
-                    "icon": QIcon()
-                }]
-                clip_choices = [{
-                    "name": "None",
-                    "value": "None",
-                    "selected": False,
-                    "icon": QIcon()
-                }]
+                tracked_choices = []
+                clip_choices = []
                 # Instantiate the timeline
                 timeline_instance = get_app().window.timeline_sync.timeline
                 # Loop through timeline's clips
-                for clip_instance in timeline_instance.Clips():
-                    clip_instance_id = clip_instance.Id()
+                for clip in Clip.filter():
+                    file_id = clip.data.get("file_id")
+
+                    # Look up parent clip id (if effect)
+                    parent_clip_id = clip_id
+                    if item_type == "effect":
+                        parent_clip_id = Effect.get(id=clip_id).parent.get("id")
+                        log.debug(f"Lookup parent clip ID for effect: '{clip_id}' = '{parent_clip_id}'")
+
                     # Avoid attach a clip to it's own object
-                    if (clip_instance_id != clip_id):
-                        # Clip's propertyJSON data
-                        clip_instance_data = Clip.get(id = clip_instance_id).data
-                        # Path to the clip file
-                        clip_instance_path = clip_instance_data["reader"]["path"]
-                        # Iterate through all clip files on the timeline
-                        for clip_number in range(self.files_model.rowCount()):
-                            clip_index = self.files_model.index(clip_number, 0)
-                            clip_name = clip_index.sibling(clip_number, 1).data()
-                            clip_path = os.path.join(clip_index.sibling(clip_number, 4).data(), clip_name)
-                            # Check if the timeline's clip file name matches the clip the user selected
-                            if (clip_path == clip_instance_path):
-                                # Generate the clip icon to show in the selection menu
-                                clip_instance_icon = clip_index.data(Qt.DecorationRole)
-                                clip_choices.append({"name": clip_instance_data["title"],
-                                              "value": clip_instance_id,
+                    if clip.id != parent_clip_id:
+                        # Iterate through all project files (to find matching QIcon)
+                        for file_index in range(self.files_model.rowCount()):
+                            file_row = self.files_model.index(file_index, 0)
+                            project_file_id = file_row.sibling(file_index, 5).data()
+                            if file_id == project_file_id:
+                                clip_instance_icon = file_row.data(Qt.DecorationRole)
+                                clip_choices.append({"name": clip.data["title"],
+                                              "value": clip.id,
                                               "selected": False,
                                               "icon": clip_instance_icon})
                         # Get the pixmap of the clip icon
@@ -590,7 +552,7 @@ class PropertiesTableView(QTableView):
                         icon_pixmap = clip_instance_icon.pixmap(icon_size, icon_size)
                         # Add tracked objects to the selection menu
                         tracked_objects = []
-                        for effect in clip_instance_data["effects"]:
+                        for effect in clip.data["effects"]:
                             # Check if effect has a tracked object
                             if effect.get("has_tracked_object"):
                                 # Instantiate the effect
@@ -610,12 +572,15 @@ class PropertiesTableView(QTableView):
                                                             "value": str(object_id),
                                                             "selected": False,
                                                             "icon": QIcon(tracked_object_icon)})
-                        tracked_choices.append({"name": clip_instance_data["title"],
-                                              "value": tracked_objects,
-                                              "selected": False,
-                                              "icon": clip_instance_icon})
-                self.choices.append({"name": _("Tracked Objects"), "value": tracked_choices, "selected": False, "icon": None})
-                self.choices.append({"name": _("Clips"), "value": clip_choices, "selected": False, "icon": None})
+                            tracked_choices.append({"name": clip.data["title"],
+                                                  "value": tracked_objects,
+                                                  "selected": False,
+                                                  "icon": clip_instance_icon})
+                self.choices.append({"name": _("None"), "value": "None", "selected": False, "icon": None})
+                if property_key == "parentObjectId" and tracked_choices:
+                    self.choices.append({"name": _("Tracked Objects"), "value": tracked_choices, "selected": False, "icon": None})
+                if clip_choices:
+                    self.choices.append({"name": _("Clips"), "value": clip_choices, "selected": False, "icon": None})
 
             # Handle reader type values
             if self.property_type == "reader" and not self.choices:
@@ -637,7 +602,8 @@ class PropertiesTableView(QTableView):
                                          })
 
                 # Add root file choice
-                self.choices.append({"name": _("Files"), "value": file_choices, "selected": False, icon: None})
+                if file_choices:
+                    self.choices.append({"name": _("Files"), "value": file_choices, "selected": False, icon: None})
 
                 # Add all transitions
                 trans_choices = []
@@ -744,6 +710,7 @@ class PropertiesTableView(QTableView):
                 menu.popup(event.globalPos())
 
             # Menu for choices
+            log.debug(f"Context menu choices: {self.choices}")
             if not self.choices:
                 return
             for choice in self.choices:
@@ -758,7 +725,7 @@ class PropertiesTableView(QTableView):
                 # Divide into smaller QMenus (since large lists cover the entire screen)
                 # For example: Transitions -> 1 -> sub items
                 SubMenu = None
-                if choice["icon"] is not None:
+                if choice.get("icon") is not None:
                     SubMenuRoot = menu.addMenu(choice["icon"], choice["name"])
                 else:
                     SubMenuRoot = menu.addMenu(choice["name"])
@@ -788,6 +755,7 @@ class PropertiesTableView(QTableView):
                         Choice_Action.triggered.connect(self.Choice_Action_Triggered)
 
             # Show choice menuk
+            log.debug(f"Display context menu: {menu.children()}")
             menu.popup(event.globalPos())
 
     def Bezier_Action_Triggered(self, preset=[]):
